--
-- Version Tagging Phase 1
-- named_version and segment_data_version_tag tables
-- version_tag field in segment_version and assignment tables
-- New trigger on segment_data_version_tag table -> segment_version
--

-- named_versions table schema
CREATE TABLE IF NOT EXISTS named_versions (
    version_tag varchar(64) not null,
    PRIMARY KEY (version_tag)
);

-- segment_data_version_tag table schema
CREATE TABLE IF NOT EXISTS segment_data_version_tag (
    segment_data_id bigint not null ,
    version_tag varchar(64) not null,
    PRIMARY KEY (segment_data_id, version_tag),
    FOREIGN KEY (segment_data_id) REFERENCES segment_data(segment_data_id) ON DELETE CASCADE,
    FOREIGN KEY (version_tag) REFERENCES named_versions(version_tag) ON DELETE CASCADE
);

-- Schema update segment_version
alter table if exists segment_version
    add column if not exists version_tag varchar(64) default 'NULL';

-- Schema update assignment
alter table if exists assignment
    add column if not exists version_tag varchar(64) default 'NULL';


-- Trigger for segment_data_version_tag -> segment_version table
do
$do$
begin
if not exists(select tgname from pg_trigger where tgname = 'trigger_segment_data_version_tag') then
    RAISE NOTICE 'Creating segment data version trigger';
    create or replace function process_segment_data_version_tag() returns trigger AS $trigger_segment_data_version_tag$
    declare
        EXISTING_SEGMENT_DATA segment_data%ROWTYPE;

        TAGGED smallint := 5;
        NA     smallint := -1;
    begin
        -- Trigger to insert or update segment_version records based on new segment_data_version_tags
        if (TG_OP = 'INSERT') then

            select * into EXISTING_SEGMENT_DATA from segment_data where segment_data_id = NEW.segment_data_id;
            IF(EXISTING_SEGMENT_DATA IS NULL) then
                raise exception 'no segment data exists';
            end if;

            -- Check if parent segment already has segment data with this version tag
            insert into segment_version (segment_version_id, segment_data_id, segment_id, platform,
                                         distribution, architecture,
                                         version_type, version_major, version_minor, version_patch,
                                         version_build, version_tag, effective_major,
                                         effective_minor, effective_patch, effective_build, created_at,
                                         updated_at, url, content_hash, content_size)
            values (default, NEW.segment_data_id, EXISTING_SEGMENT_DATA.segment_id, EXISTING_SEGMENT_DATA.platform,
                    EXISTING_SEGMENT_DATA.distribution,
                    EXISTING_SEGMENT_DATA.architecture, TAGGED,
                    NA, NA, NA, NA, NEW.version_tag, EXISTING_SEGMENT_DATA.major, EXISTING_SEGMENT_DATA.minor, EXISTING_SEGMENT_DATA.patch, EXISTING_SEGMENT_DATA.build,
                    EXISTING_SEGMENT_DATA.created_at,
                    EXISTING_SEGMENT_DATA.updated_at, EXISTING_SEGMENT_DATA.url, EXISTING_SEGMENT_DATA.content_hash,
                    EXISTING_SEGMENT_DATA.content_size);
        end if;
        RETURN NULL; -- result is ignored since this is an AFTER trigger
    end;
    $trigger_segment_data_version_tag$ language plpgsql;

    create trigger trigger_segment_data_version_tag
        after insert
        on segment_data_version_tag
        for each row
    execute procedure process_segment_data_version_tag();
else
    raise notice 'Segment Data Version Tag already exists, skipping.';
end if;
end
$do$;