-- DSER-37404 Add processor, aggregate, and aggregator fields to the segment table


-- Reference table for the PROCESSOR enumeration
create table if not exists processor (
                                         id smallint not null,
                                         name varchar(50) not null,
                                         urn varchar(255) not null,
                                         description varchar(255) not null,
                                         primary key (id)
);

COMMENT ON TABLE processor IS 'Processor is used to specify how segment data is pre-processed prior to persistence';

insert into processor
(id, name, urn, description)
values (0, 'STATIC', 'psc:content:processor:static', 'No pre-processing needed')
on conflict do nothing;

------------------------------------------------------------------------------------------------------------
-- Reference table for the AGGREGATE enumeration
create table if not exists  aggregate (
                                          id smallint not null,
                                          name varchar(50) not null,
                                          description varchar(200) not null,
                                          primary key (id)
);

COMMENT ON TABLE aggregate IS 'Aggregate is used to denote what will be aggregated for a collection of segments';

insert into aggregate
(id, name, description)
values (0, 'SEGMENT_TYPE', 'Segments must be grouped into LIKE segment types when delivered to the consumer')
on conflict do nothing;

insert into aggregate
(id, name, description)
values (1, 'SEGMENT', 'The Segment is standalone content that must be delivered individually')
on conflict do nothing;

insert into aggregate
(id, name, description)
values (2, 'CONTENT_TYPE', 'The Segments must be grouped into LIKE content types when delivered to the consumer')
on conflict do nothing;

------------------------------------------------------------------------------------------------------------
-- Reference table for the AGGREGATOR enumeration
create table if not exists  aggregator (
                                           id smallint not null,
                                           name varchar(50) not null,
                                           description varchar(200) not null,
                                           primary key (id)
);

COMMENT ON TABLE aggregator IS 'Aggregator is is used to denote how the collection of segments will be aggregated when delivered to the consumer';

insert into aggregator
(id, name, description)
values (0, 'RULE_JSON', 'The content must be aggregated in the proper JSON format according to DRE syntax')
on conflict do nothing;

insert into aggregator
(id, name, description)
values (1, 'NONE', 'There is no aggregation needed and that we simply return stored content')
on conflict do nothing;

------------------------------------------------------------------------------------------------------------

-- Sensor rules
-- processor  - default is 0 (STATIC)
-- aggregate  - default is 0 (SEGMENT_TYPE)
-- aggregator - default is 0 (RULE_JSON)

-- Sensor Kits & UDS Segment types
-- processor  - default is 0 (STATIC) --- SAME for Sensor
-- aggregate  - default is 1 (SEGMENT)
-- aggregator - default is 1 (NONE)

begin transaction;

alter table if exists segment
    add column if not exists processor smallint references processor(id),
    add column if not exists aggregate smallint references aggregate(id),
    add column if not exists aggregator smallint references aggregator(id);

alter table if exists segment_type
    add column if not exists supported_aggregators varchar(255);

COMMENT ON column segment_type.supported_aggregators IS 'supported_aggregators is used to enforce which aggregation behavior segment types support';

-- Update all segments without the sensor kits and UDS_BUNDLE segment types
-- Commented out as this was a one time migration

-- update segment
-- set processor=0, aggregate=0, aggregator=0 where segment_type not in (11,40);

-- Update the SENSOR_KITS and UDS_BUNDLE segment types to have the appropriate defaults
-- segment_type 11 = SENSOR_KITS
-- segment_type 40 = UDS_BUNDLE
-- Commented out as this was a one time migration

-- update segment
-- set processor=0, aggregate=1, aggregator=1 where segment_type in (11,40);

-- Disallow SENSOR_KITS from adding any functional aggregator
-- Disallow DREC segment types (UDS_BUNDLE) from adding any functional aggregator
update segment_type
set supported_aggregators='NONE' where segment_type_id in (11,40);

update segment_type
set supported_aggregators='RULE_JSON' where segment_type_id not in (11,40) and format=0;

-- Add constraints to columns to make sure that nothing can be added without these columns populated
alter table segment alter column processor set not null;
alter table segment alter column aggregate set not null;
alter table segment alter column aggregator set not null;
alter table segment_type alter column supported_aggregators set not null;

-- The UDS segment types content type needs to change to the content_type 3 (CLOUD_RULE)
update segment_type set content_type = 3 where name ='UDS_BUNDLE' or name = 'UDS_PIPELINE';

end;

